import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import update from 'immutability-helper'
import { message, Empty } from 'antd'

import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const Container = ({config, editId, handleList, editCard, deleteCard, doubleClickCard }) => {
  let target = null

  const [cards, setCards] = useState(config.components)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    handleList({...config, components: _cards})
  }

  if (!is(fromJS(cards), fromJS(config.components))) {
    setCards(config.components)
  }
  
  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const hasDrop = (item) => {
    target = item
  }

  const [, drop] = useDrop({
    accept: 'mob',
    drop(item) {
      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      if (cards.length > 0 && cards[0].type === 'login') {
        message.warning('登录页不可添加其他元素！')
        return
      }

      let newcard = fromJS(item.param).toJS()
      Object.keys(newcard).forEach(key => {
        if (newcard[key].editable) {
          newcard[key].uuid = Utils.getuuid()
        }
      })
      newcard.uuid = Utils.getuuid()
      
      let targetId = cards.length > 0 ? cards[cards.length - 1].uuid : 0
      if (target) {
        targetId = target.uuid
      }

      const { index: overIndex } = findCard(`${targetId}`)
      let targetIndex = overIndex

      targetIndex++

      const _cards = update(cards, { $splice: [[targetIndex, 0, newcard]] })

      handleList({...config, components: _cards})
      target = null
    }
  })

  return (
    <div ref={drop} className="mob-shell-inner">
      {cards.map(card => (
        <Card
          id={card.uuid}
          key={card.uuid}
          card={card}
          editId={editId}
          moveCard={moveCard}
          editCard={editCard}
          delCard={deleteCard}
          findCard={findCard}
          hasDrop={hasDrop}
          doubleClickCard={doubleClickCard}
        />
      ))}
      {cards.length === 0 ?
        <Empty description="请添加组件" /> : null
      }
    </div>
  )
}
export default Container
